/*
 * Copyright (c) 2016 Intel Corporation.
 *
 * SPDX-License-Identifier: Apache-2.0
 */

#include <zephyr.h>
#include <drivers/flash.h>
#include <device.h>
#include <devicetree.h>
#include <stdio.h>
#include <string.h>

#define FLASH_DEVICE DT_LABEL(DT_INST(0, nordic_qspicustom))
#define FLASH_NAME "JEDEC QSPI-NOR"

#define FLASH_TEST_REGION_OFFSET 0xff000
#define FLASH_SECTOR_SIZE 16384

#define FLASH_BUF_SIZE 256
#define TEST_READ_LEN 2
uint8_t flash_buf[256];

void read(const struct device *flash_dev)
{
	int rc;
	// Read the sector we just wrote and measure the time it took
	// The flash_read function will yield to the compute thread, then preempt it when the flash
	// read is done
	rc = flash_read(flash_dev, 0, flash_buf, TEST_READ_LEN);
	if (rc != 0)
	{
		printk("Flash read failed! %d\n", rc);
		while (true)
			;
	}

	printk("Flash read success\r\n");
	for (uint16_t i = 0; i < TEST_READ_LEN; i++)
	{
		printk("0x%x ", flash_buf[i]);
	}
	printk("\r\n");
}

void main(void)
{
	const struct device *flash_dev;

	printf("\n" FLASH_NAME " SPI flash testing\n");
	printf("==========================\n");

	flash_dev = device_get_binding(FLASH_DEVICE);

	if (!flash_dev)
	{
		printf("SPI flash driver %s was not found!\n",
			   FLASH_DEVICE);
		return;
	}

	memset(flash_buf, 0, FLASH_BUF_SIZE);

	printk("Starting flash read loop...\n");
	k_msleep(25);

	read(flash_dev);
	k_msleep(25);
	read(flash_dev);
	k_sleep(K_FOREVER);

	while (1)
		;
}
